/*
 * AutoDAO - Generic DAO on steroids implementation for Java.
 *
 * Copyright 2008-2012  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.autodao.impl;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Map;

import net.sf.autodao.QueryArgumentTransformer;
import org.jetbrains.annotations.NotNull;
import org.joda.time.DateMidnight;
import org.joda.time.DateTime;
import org.joda.time.Instant;
import org.joda.time.MutableDateTime;
import org.joda.time.ReadableInstant;

/**
 * Handles Joda-Time {@link ReadableInstant} subclasses and transforms them into {@link Calendar}.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
final class ReadableInstantTransformer implements QueryArgumentTransformer<ReadableInstant, Calendar> {

  @SuppressWarnings({ "unchecked" })
  static void register(@NotNull final Map<Class<?>, QueryArgumentTransformer> map) {
    final QueryArgumentTransformer<?, ?> jodaTimeTransformer = new ReadableInstantTransformer();
    for (final Class<?> impl : Arrays.asList(Instant.class, MutableDateTime.class, DateTime.class, DateMidnight.class)) {
      Class<?> klass = impl;
      while (ReadableInstant.class.isAssignableFrom(klass)) {
        map.put(klass, jodaTimeTransformer);
        for (final Class<?> interfaze : klass.getInterfaces())
          if (ReadableInstant.class.isAssignableFrom(interfaze))
            map.put(interfaze, jodaTimeTransformer);

        klass = klass.getSuperclass();
      }
    }
  }

  @NotNull
  @Override
  public Calendar transform(@NotNull final ReadableInstant argument) {
    final Calendar c = Calendar.getInstance();
    c.setTimeInMillis(argument.getMillis());
    return c;
  }

  @NotNull
  @Override
  public Class<Calendar> getTargetType() {
    return Calendar.class;
  }
}
