/*
 * AutoDAO - Generic DAO on steroids implementation for Java.
 *
 * Copyright 2008-2012  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.autodao.impl.jpa;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Parameter;
import javax.persistence.Query;

import net.sf.autodao.Dao;
import net.sf.autodao.PersistentEntity;
import net.sf.autodao.QueryArgumentTransformer;
import net.sf.autodao.impl.AbstractDaoFactoryBean;
import net.sf.autodao.impl.FinderExecutor;
import net.sf.autodao.impl.ParametersChecker;
import net.sf.autodao.impl.SpecificationViolationException;
import net.sf.autodao.impl.Utils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.util.Assert;
import org.springframework.util.ReflectionUtils;

/**
 * JPA DAO proxy factory.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
public final class JpaDaoProxyFactoryBean extends AbstractDaoFactoryBean {

  @NotNull
  private final EntityManagerFactory entityManagerFactory;

  protected JpaDaoProxyFactoryBean(@NotNull Class<?> daoInterface,
                                   @Nullable Class<? extends PersistentEntity<?>> entityType,
                                   @NotNull PlatformTransactionManager transactionManager,
                                   @NotNull EntityManagerFactory entityManagerFactory) {
    super(daoInterface, entityType, transactionManager);
    this.entityManagerFactory = entityManagerFactory;
  }

  @SuppressWarnings({ "unchecked" })
  @NotNull
  protected FinderExecutor createDao(@Nullable final Class<?> entityType) {
    if (entityType == null) {
      return new JpaDao(entityManagerFactory);
    } else {

      final EntityManager entityManager = entityManagerFactory.createEntityManager();
      try {
        Assert.notNull(entityManager.getMetamodel().entity(entityType));
      } finally {
        entityManager.close();
      }

      return new JpaEntityDao(entityType, entityManagerFactory);
    }
  }

  /** Validates DAO interface. */
  protected void validate(@NotNull final Class<?> daoInterface, @NotNull final Map<Class<?>, QueryArgumentTransformer> transformers) {
    ReflectionUtils.doWithMethods(daoInterface, new ReflectionUtils.MethodCallback() {
      @Override
      public void doWith(final Method method) {
        if (method.getDeclaringClass().equals(Dao.class))
          return;

        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        try {
          final Query query = Utils.visitQuery(method, new Utils.QueryVisitor<Query>() {
            @Override
            public Query visitNamedQuery(@NotNull String queryName) {
              try {
                return entityManager.createNamedQuery(queryName);
              } catch (final IllegalArgumentException e) {
                throw new SpecificationViolationException("No named query '" + queryName + "' found.", method);
              }
            }

            @Override
            public Query visitQuery(@NotNull String query) {
              try {
                return entityManager.createQuery(query);
              } catch (final IllegalArgumentException e) {
                throw new SpecificationViolationException("Invalid query: " + query, method, e);
              }
            }

            @Override
            public Query visitSQLNamedQuery(@NotNull String queryName) {
              try {
                return entityManager.createNamedQuery(queryName);
              } catch (final IllegalArgumentException e) {
                throw new SpecificationViolationException("No named query '" + queryName + "' found.", method);
              }
            }

            @Override
            public Query visitSQLQuery(@NotNull String query) {
              try {
                return entityManager.createNativeQuery(query);
              } catch (final IllegalArgumentException e) {
                throw new SpecificationViolationException("Invalid query: " + query, method, e);
              }
            }
          });

          int ordinalTmp = 0;
          final Set<String> named = new HashSet<String>();
          for (Parameter p : query.getParameters()) {
            if (p.getName() == null)
              ordinalTmp++;
            else
              named.add(p.getName());
          }
          final int ordinal = ordinalTmp;

          new ParametersChecker(method, transformers) {
            @Override
            protected Class<?> getExpectedType(int paramIndex) {
              return query.getParameter(paramIndex).getParameterType();
            }

            @Override
            protected int getExpectedOrdinalParametersCount() {
              return ordinal;
            }

            @Override
            protected Set<String> getExpectedNamedParameters() {
              return named;
            }

            @Override
            protected Class<?> getExpectedType(@NotNull String paramName) {
              return query.getParameter(paramName).getParameterType();
            }
          }.check();
        } finally {
          entityManager.close();
        }
      }
    });
  }
}
